# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# source: grpc.proto
"""Generated protocol buffer code."""
from google.protobuf import descriptor as _descriptor
from google.protobuf import descriptor_pool as _descriptor_pool
from google.protobuf import message as _message
from google.protobuf import reflection as _reflection
from google.protobuf import symbol_database as _symbol_database
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()




DESCRIPTOR = _descriptor_pool.Default().AddSerializedFile(b'\n\ngrpc.proto\"3\n\x0cHelloRequest\x12\x0c\n\x04\x64\x61ta\x18\x01 \x01(\t\x12\x15\n\x05skill\x18\x02 \x01(\x0b\x32\x06.Skill\"\x84\x01\n\rHelloResponse\x12\x0e\n\x06result\x18\x01 \x01(\t\x12\x31\n\nmap_result\x18\x02 \x03(\x0b\x32\x1d.HelloResponse.MapResultEntry\x1a\x30\n\x0eMapResultEntry\x12\x0b\n\x03key\x18\x01 \x01(\t\x12\r\n\x05value\x18\x02 \x01(\x05:\x02\x38\x01\"\x15\n\x05Skill\x12\x0c\n\x04name\x18\x01 \x01(\t\"+\n\x0b\x43\x61llRequest\x12\x0c\n\x04\x66unc\x18\x01 \x01(\t\x12\x0e\n\x06params\x18\x02 \x01(\x0c\",\n\x0c\x43\x61llResponse\x12\x0e\n\x06status\x18\x01 \x01(\x05\x12\x0c\n\x04\x64\x61ta\x18\x02 \x01(\x0c\"\xa4\x01\n\x06Params\x12\x1d\n\x03key\x18\x01 \x03(\x0b\x32\x10.Params.KeyEntry\x12!\n\x05value\x18\x02 \x03(\x0b\x32\x12.Params.ValueEntry\x1a*\n\x08KeyEntry\x12\x0b\n\x03key\x18\x01 \x01(\t\x12\r\n\x05value\x18\x02 \x01(\t:\x02\x38\x01\x1a,\n\nValueEntry\x12\x0b\n\x03key\x18\x01 \x01(\t\x12\r\n\x05value\x18\x02 \x01(\x05:\x02\x38\x01\x32^\n\x0bGrpcService\x12(\n\x05hello\x12\r.HelloRequest\x1a\x0e.HelloResponse\"\x00\x12%\n\x04\x63\x61ll\x12\x0c.CallRequest\x1a\r.CallResponse\"\x00\x42\x34\n\x1c\x63om.wxrh.cloud.grpc.sayHelloB\x0fHelloWorldProtoP\x01\x80\x01\x01\x62\x06proto3')



_HELLOREQUEST = DESCRIPTOR.message_types_by_name['HelloRequest']
_HELLORESPONSE = DESCRIPTOR.message_types_by_name['HelloResponse']
_HELLORESPONSE_MAPRESULTENTRY = _HELLORESPONSE.nested_types_by_name['MapResultEntry']
_SKILL = DESCRIPTOR.message_types_by_name['Skill']
_CALLREQUEST = DESCRIPTOR.message_types_by_name['CallRequest']
_CALLRESPONSE = DESCRIPTOR.message_types_by_name['CallResponse']
_PARAMS = DESCRIPTOR.message_types_by_name['Params']
_PARAMS_KEYENTRY = _PARAMS.nested_types_by_name['KeyEntry']
_PARAMS_VALUEENTRY = _PARAMS.nested_types_by_name['ValueEntry']
HelloRequest = _reflection.GeneratedProtocolMessageType('HelloRequest', (_message.Message,), {
  'DESCRIPTOR' : _HELLOREQUEST,
  '__module__' : 'grpc_pb2'
  # @@protoc_insertion_point(class_scope:HelloRequest)
  })
_sym_db.RegisterMessage(HelloRequest)

HelloResponse = _reflection.GeneratedProtocolMessageType('HelloResponse', (_message.Message,), {

  'MapResultEntry' : _reflection.GeneratedProtocolMessageType('MapResultEntry', (_message.Message,), {
    'DESCRIPTOR' : _HELLORESPONSE_MAPRESULTENTRY,
    '__module__' : 'grpc_pb2'
    # @@protoc_insertion_point(class_scope:HelloResponse.MapResultEntry)
    })
  ,
  'DESCRIPTOR' : _HELLORESPONSE,
  '__module__' : 'grpc_pb2'
  # @@protoc_insertion_point(class_scope:HelloResponse)
  })
_sym_db.RegisterMessage(HelloResponse)
_sym_db.RegisterMessage(HelloResponse.MapResultEntry)

Skill = _reflection.GeneratedProtocolMessageType('Skill', (_message.Message,), {
  'DESCRIPTOR' : _SKILL,
  '__module__' : 'grpc_pb2'
  # @@protoc_insertion_point(class_scope:Skill)
  })
_sym_db.RegisterMessage(Skill)

CallRequest = _reflection.GeneratedProtocolMessageType('CallRequest', (_message.Message,), {
  'DESCRIPTOR' : _CALLREQUEST,
  '__module__' : 'grpc_pb2'
  # @@protoc_insertion_point(class_scope:CallRequest)
  })
_sym_db.RegisterMessage(CallRequest)

CallResponse = _reflection.GeneratedProtocolMessageType('CallResponse', (_message.Message,), {
  'DESCRIPTOR' : _CALLRESPONSE,
  '__module__' : 'grpc_pb2'
  # @@protoc_insertion_point(class_scope:CallResponse)
  })
_sym_db.RegisterMessage(CallResponse)

Params = _reflection.GeneratedProtocolMessageType('Params', (_message.Message,), {

  'KeyEntry' : _reflection.GeneratedProtocolMessageType('KeyEntry', (_message.Message,), {
    'DESCRIPTOR' : _PARAMS_KEYENTRY,
    '__module__' : 'grpc_pb2'
    # @@protoc_insertion_point(class_scope:Params.KeyEntry)
    })
  ,

  'ValueEntry' : _reflection.GeneratedProtocolMessageType('ValueEntry', (_message.Message,), {
    'DESCRIPTOR' : _PARAMS_VALUEENTRY,
    '__module__' : 'grpc_pb2'
    # @@protoc_insertion_point(class_scope:Params.ValueEntry)
    })
  ,
  'DESCRIPTOR' : _PARAMS,
  '__module__' : 'grpc_pb2'
  # @@protoc_insertion_point(class_scope:Params)
  })
_sym_db.RegisterMessage(Params)
_sym_db.RegisterMessage(Params.KeyEntry)
_sym_db.RegisterMessage(Params.ValueEntry)

_GRPCSERVICE = DESCRIPTOR.services_by_name['GrpcService']
if _descriptor._USE_C_DESCRIPTORS == False:

  DESCRIPTOR._options = None
  DESCRIPTOR._serialized_options = b'\n\034com.wxrh.cloud.grpc.sayHelloB\017HelloWorldProtoP\001\200\001\001'
  _HELLORESPONSE_MAPRESULTENTRY._options = None
  _HELLORESPONSE_MAPRESULTENTRY._serialized_options = b'8\001'
  _PARAMS_KEYENTRY._options = None
  _PARAMS_KEYENTRY._serialized_options = b'8\001'
  _PARAMS_VALUEENTRY._options = None
  _PARAMS_VALUEENTRY._serialized_options = b'8\001'
  _HELLOREQUEST._serialized_start=14
  _HELLOREQUEST._serialized_end=65
  _HELLORESPONSE._serialized_start=68
  _HELLORESPONSE._serialized_end=200
  _HELLORESPONSE_MAPRESULTENTRY._serialized_start=152
  _HELLORESPONSE_MAPRESULTENTRY._serialized_end=200
  _SKILL._serialized_start=202
  _SKILL._serialized_end=223
  _CALLREQUEST._serialized_start=225
  _CALLREQUEST._serialized_end=268
  _CALLRESPONSE._serialized_start=270
  _CALLRESPONSE._serialized_end=314
  _PARAMS._serialized_start=317
  _PARAMS._serialized_end=481
  _PARAMS_KEYENTRY._serialized_start=393
  _PARAMS_KEYENTRY._serialized_end=435
  _PARAMS_VALUEENTRY._serialized_start=437
  _PARAMS_VALUEENTRY._serialized_end=481
  _GRPCSERVICE._serialized_start=483
  _GRPCSERVICE._serialized_end=577
# @@protoc_insertion_point(module_scope)
